# coding: utf-8
import re
import subprocess
import requests
from sunday.core.logger import Logger

logger = Logger('checkPacNet').getLogger()

def checkWifi(name):
    '''传入wifi名, 返回当前wifi是否匹配'''
    if not name: return False
    if type(name) == str: name = [name]
    logger.debug('检查wifi: %s' % '、'.join(name))
    command = ['/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport', '-I']
    res = subprocess.check_output(command).decode('utf-8')
    return any(re.search(re.compile('SSID: %s\n' % item), res) for item in name)

def checkUrl(url):
    '''传入链接名, 返回当前网络是否能访问到'''
    if not url: return False
    if type(url) == str: url = [url]
    logger.debug('检查链接: %s' % '、'.join(url))
    def requestUrl(tar):
        try:
            return requests.get(tar).ok
        except:
            return False
    return any(requestUrl(item) for item in url)

def checkPacNet(wifi, url):
    return checkWifi(wifi) or checkUrl(url)



# coding: utf-8
import os
import sys
from dotenv import load_dotenv, set_key, unset_key
from sunday.core.logger import Logger

logger = Logger('enver').getLogger()

def enver(file):
    '''传入env文件, 返回操作env文件的方法'''
    if not os.path.exists(os.path.dirname(file)):
        logger.error('路径目录不存在 %s' % file)
        sys.exit(1)
    if not os.path.isfile(file):
        logger.debug('新建文件 %s' % file)
        os.system('touch %s' % file)
    load_dotenv(file)
    def getenv(key):
        '''返回env中对应key的值'''
        return os.environ.get(str(key))
    def setenv(key, val = None):
        '''设置env中对应key的值'''
        if not val:
            os.environ.pop(key)
            unset_key(file, str(key))
        else:
            set_key(file, str(key), str(val))
    def getfile():
        return file
    return getenv, setenv, getfile



# coding: utf-8
import logging
import colorlog
import atexit
import datetime
from os import path
from sunday.core.globalvar import getvar, setvar
from sunday.core.getConfig import getConfig
import sunday.core.paths as paths

if not getvar('loggerId'):
    loggerId = datetime.datetime.today().isoformat()
    setvar('loggerId', loggerId)
    if getConfig('LOGGING')('print_file'):
        @atexit.register
        def exitPrintLogfile():
            print('\n    LOG FILE AT: %s' % path.join(paths.logCwd, loggerId))

logfile = path.join(paths.logCwd, getvar('loggerId'))

logging.basicConfig(filename=logfile, level=logging.DEBUG, force=True,
        format='[%(asctime)s.%(msecs)-3d] %(levelname)s <%(name)s>: %(message)s')

class Logger():
    def __init__(self, name, level='', format=''):
        cfg = getConfig('LOGGING')
        logger = logging.getLogger(name)
        logger.setLevel(logging.getLevelName(level or cfg('level')))
        loghd = logging.StreamHandler()
        loghd.setFormatter(colorlog.ColoredFormatter(format or cfg('format'), datefmt='%H:%M:%S'))
        logger.addHandler(loghd)
        logger.propagate = True
        self.logger = logger

    def getLogger(self):
        return self.logger



# coding: utf-8
import subprocess
from threading import Timer
from sunday.core.logger import Logger

logger = Logger('cmdexec').getLogger()

""" subprocess.Popen
入参: args, bufsize=0, executable=None, stdin=None, stdout=None, stderr=None,
preexec_fn=None, close_fds=False, shell=False, cwd=None, env=None,
universal_newlines=False, startupinfo=None, creationflags=0
出参: stdin, stdout, stderr, pid, returncode
"""

def cmdexec(cmd, timeout = 10, **kwargs):
    """ 执行系统命令, 返回: code, stdout, stderr, 当code不为0则为报错"""
    subp = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, **kwargs)
    timer = Timer(timeout, lambda p: p.kill(), [subp])
    try:
        timer.start()
        subp.wait()
        code = subp.returncode
        stdout = subp.stdout.read()
        stderr = subp.stderr.read()
        getattr(logger, 'error' if code else 'info')('%d: %s' % (code, cmd))
        return code, stdout.decode(), stderr.decode()
    except Exception as e:
        return 1, '', 'timeout'
    finally:
        timer.cancel()


if __name__ == "__main__":
    cmdexec('getToken')



# coding: utf-8
import requests
import sunday.core.paths as paths
from http.cookiejar import LWPCookieJar
from sunday.core.getEnv import getEnv
from sunday.core.logger import Logger
from sunday.core.common import exit
from sunday.core.checkPacNet import checkPacNet
from sunday.core.getConfig import getConfig
from sunday.core.enver import enver
from sunday.core.auth import Auth, getCurrentUser
from pypac import PACSession
from requests.auth import HTTPProxyAuth

# 关闭由verify=False引起的提示
requests.packages.urllib3.disable_warnings()

headers = {
    'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_6) '
        'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36'),
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache',
    'Accept-Encoding': 'gzip, deflate, br',
    'Accept-Language': 'zh-CN,zh;q=0.9'
}

headers_post = {
    'contentType': {
        'Content-Type': 'application/json;charset=UTF-8'
    }
}

logger = Logger('Fetch').getLogger()
class Fetch():
    def __init__(self, pacWifi = None, pacUrl = None, pacWifiName = None):
        # pacWifiName 兼容老写法
        self.pacWifi = pacWifi or pacWifiName
        self.pacUrl = pacUrl
        self._hasProxy = False
        self._hasPacProxy = False
        self.session = self.get_session()

    def get_session(self):
        # 返回session
        proxy = getConfig('PROXY')('proxy')
        session = requests.session() if proxy else PACSession()
        self.openProxy(session, proxy)
        session.headers.update(headers)
        session.verify = False
        return session
    
    def openProxy(self, session, proxy):
        '''开启代理'''
        if proxy:
            logger.warning('存在代理%s' % proxy)
            session.proxies = {
                'http': proxy,
                'https': proxy
            }
            self._hasProxy = True
        else:
            self.openPacProxy(session)
    
    def add_header(self, headers):
        '''更新请求头'''
        if type(headers) == dict:
            self.session.headers.update(headers)
    
    def closeProxy(self):
        '''关闭代理'''
        self.session.proxies = None
        self._hasProxy = False
    
    def hasProxy(self):
        '''返回是否开启代理'''
        return self._hasProxy

    def hasPacProxy(self):
        '''返回是否开启pac'''
        return self._hasPacProxy

    def openPacProxy(self, session):
        '''检测为内网环境则增加代理认证'''
        if checkPacNet(self.pacWifi, self.pacUrl):
            auth = Auth(paths.envCwd, '[PAC PROXY]')
            getenv = enver(paths.envCwd)[0]
            username = auth.addParams('pac_user', 'PAC代理账号', value=getenv('pac_user'), defaultValue=getCurrentUser())
            password = auth.addParams('pac_pass', 'PAC代理密码', value=getenv('pac_pass'), isPass=True)
            logger.warning('pac认证网络, 网络交互使用代理中')
            session.proxy_auth = HTTPProxyAuth(username, password)
            self._hasPacProxy = True
            return True
        return False

    def requestByType(self, type, times, *args, **kwargs):
        try:
            res = getattr(self.session, type)(*args, **kwargs)
            logger.info('fetch result %s:%d' % (res.url, res.status_code))
            return res
        except Exception as e:
            if times >= 3:
                exit('第%d次尝试失败, 请检查网络后重试!' % times)
            times += 1
            logger.warning('接口请求失败, 进行第%d次尝试: %s' % (times, e))
            return self.requestByType(type, times, *args, **kwargs)
    
    def get(self, *args, **kwargs):
        return self.requestByType('get', 0, *args, **kwargs)

    def post(self, *args, **kwargs):
        return self.requestByType('post', 0, *args, **kwargs)
    
    def getCookiesDict(self):
        '''cookie转换成对象后返回'''
        cookies = self.session.cookies
        if 'get_dict' in cookies:
            return cookies.get_dict()
        return requests.utils.dict_from_cookiejar(cookies)



# coding: utf-8
import os
import sys
import re

def clear():
    """清理屏幕"""
    os.system('clear')

def exit(str = ''):
    """退出程序"""
    if str: print(str)
    sys.exit(1)

def pascal_to_snake(camel_case):
    """驼峰转蛇形"""
    snake_case = re.sub(r"(?P<key>[A-Z])", r"_\g<key>", camel_case)
    return snake_case.lower().strip('_')

def snake_to_pascal(snake_case):
    """蛇形转驼峰"""
    words = snake_case.split('_')
    return ''.join(word.title() for word in words)


# coding: utf-8
import os
import types

def mergeObj(ori={}, *objs):
    '''合并字典'''
    newObj = ori.copy()
    for obj in objs:
        newObj.update(obj)
    return newObj


def get_format_str(widths):
    str_width = sum(widths) + (len(widths) - 1) * 3 + 4
    ans = '|'
    for width in widths:
        ans += ' %-{}s'.format(width)
    return ans, str_width

def get_width(ans):
    width = 0
    times = 0
    if type(ans) in [bytes, str]:
        for s in ans:
            if ord(s) > 127:
                times += 1
        width = len(ans) + times
    return width, times



# coding: utf-8
import argparse
from pydash import pick, omit, get
from .common import pascal_to_snake

arr = ['version', 'params']

def formatter(prog):
    return argparse.RawDescriptionHelpFormatter(prog, indent_increment=2, max_help_position=100, width=140)

def getParserDefault(config, name='DEFAULT'):
    '''
    用于返回命令入参的默认值对象
        config<dict>: 为命令入参
        name<str>: 默认为DEAFULT, 如果返回指定子命令的默认值, 可替代为子命令名称
    '''
    arr = get(config, '.'.join(['params', name])) or []
    obj = {}
    for item in arr:
        obj[item['dest']] = get(item, 'default')
    return obj

def getParser(**argvs):
    '''
    传入对象, 除以下参数, 其它参数用于实例化parser使用;
        1. version<str>: 程序版本
        2. params<dict>: 命令入参
            params.DEFAULT<list>: 默认命令入参
            params.SUBCONFIG<dict>: 子命令配置
            params[name]<list>: 名称为name的子命令的命令入参
        实例化用如: description、epilog等, 具体可参考argparse模块
    '''
    parserObj, commandObj = omit(argvs, arr), pick(argvs, arr)
    parser = argparse.ArgumentParser(
        add_help=False,
        formatter_class=formatter,
        **parserObj
    )
    parser._positionals.title = 'Positionals'
    parser._optionals.title = 'Optional'
    parser.add_argument('-v', '--version', action='version', version=commandObj['version'] or '0.0.0', help='当前程序版本')
    parser.add_argument('-h', '--help', action='help', default=argparse.SUPPRESS, help='打印帮助说明')
    subparsers = None
    subparsersObj = {}
    if 'params' in commandObj and type(commandObj['params']) == dict:
        paramsCfg = commandObj['params']
        if 'SUBCONFIG' in paramsCfg and type(paramsCfg['SUBCONFIG']) == dict:
            subparsers = parser.add_subparsers(**paramsCfg['SUBCONFIG'])
        for (name, cfgs) in list(commandObj['params'].items()):
            tarparser = None
            if name == 'SUBCONFIG' or type(cfgs) != list:
                continue
            elif name == 'DEFAULT':
                tarparser = parser
            elif subparsers:
                tarparser = subparsers.add_parser(name)
                subparsersObj[name] = tarparser
            else:
                continue
            for cfg in cfgs:
                tar = omit(cfg, ['name'])
                if 'metavar' not in cfg and get(tar, 'action') not in ['store_false', 'store_true']:
                    tar['metavar'] = pascal_to_snake(cfg['dest']).upper()
                tarparser.add_argument(*cfg['name'], **tar)
    # 兼容老用法
    if not subparsers: return parser
    return parser, subparsersObj



# coding: utf-8
import os
from getpass import getpass
from sunday.core.logger import Logger
from sunday.core.getEnv import getEnv
from sunday.core.getConfig import getConfig
from sunday.core.aesCbc import aesCbcDecrypt, aesCbcEncrypt

__all__ = ['getCurrentUser', 'Auth']

logger = Logger('AUTH').getLogger()

def getCurrentUser():
    return getEnv('USER')

cryptoKey = getConfig('CRYPTO')('key')

class Auth():
    """ 用于用户账户认证相关操作
    attributes:
        addParams: 用户名密码等交互元素录入
        getParams: 返回交互结果对象
    examples:
        from utils.auth import Auth
        auth = Auth('path/to/.env')
        auth.addParams('USER')
        auth.addParams('USER', 'ask text', defaultValue="xxx")
        auth.addParams('USER', 'ask text', 'myName')
        auth.addParams('code', isSave=False)
        params = auth.getParams()
    raises:
        请输入env文件路径: 实例化时envPath必传
    """
    def __init__(self, envPath = '', name = ''):
        if not envPath: raise Exception('请输入env文件路径')
        self.name = name
        self.envPath = envPath
        self.tip = {}
        self.val = {}

    def tipMap(self, key):
        tip = key
        if key in ['USER', 'user', 'username', 'USERNAME', 'name', 'NAME']:
            tip = '请输入用户名'
        elif key in ['PWD', 'pwd', 'PASSWORD', 'password']:
            tip = '请输入密码'
        elif key in ['CODE', 'code']:
            tip = '请输入验证码'
        return self.name + tip

    def addParams(self, key, tip = '', value='', defaultValue='', isMust=True, isSave=True, isPass=False):
        """ 用户名密码等交互元素录入
        Args:
            key: 交互元素键名
            tip: 交互时的提示文本
            value: 值存在则跳过交互
            defaultValue: 默认值, 当输入为空时取默认值
            isMust: 是否必填
            isSave: 是否缓存, 用户名密码一般需要缓存, 验证码一般不需要缓存
        Returns:
            value存在则返回value否则返回用户交互结果
        """
        self.tip[key] = self.tipMap(tip or key)
        tarVal = value
        if not value:
            tarVal = self.ask(key, defaultValue, isMust, isSave, isPass)
        elif isPass:
            tarVal = aesCbcDecrypt(value, cryptoKey)
        self.val[key] = tarVal
        return self.val[key]

    def ask(self, key, defaultValue, isMust, isSave, isPass):
        tip = '%s%s: ' % (self.tip[key], '(%s)' % defaultValue if defaultValue else '')
        value = getpass(tip) if isPass else input(tip)
        if value is '':
            if defaultValue:
                value = defaultValue
            elif isMust:
                print('必填项不能为空!')
                return self.ask(key, defaultValue, isMust, isSave)
        if isSave:
            # isPass则存储密文, 否则存储明文
            tarVal = aesCbcEncrypt(value, cryptoKey) if isPass else value
            cmd = 'echo %s=%s >> %s' % (key, tarVal, self.envPath)
            os.system(cmd)
        return value

    def getParams(self):
        return self.val


if __name__ == '__main__':
    auth = Auth()
    auth.addParams('user', '哈哈')
